package de.fraunhofer.iem.dataprovider.tool.service

import de.fraunhofer.iem.dataprovider.tool.dto.CreateToolDto
import de.fraunhofer.iem.dataprovider.tool.entity.ToolEntity
import de.fraunhofer.iem.dataprovider.tool.repository.ToolRepository
import de.fraunhofer.iem.dataprovider.tools.gitlab.service.RepositoryDetailsService
import de.fraunhofer.iem.dataprovider.tools.occmd.service.OccmdService
import de.fraunhofer.iem.dataprovider.tools.ort.service.OrtService
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional

@Service
class ToolService(
    private val toolRepository: ToolRepository,
    private val ortService: OrtService,
    private val occmdService: OccmdService,
    private val repositoryDetailsService: RepositoryDetailsService
) {

    fun createAllTools() {
        findOrCreateTool(ortService.getToolDto())
        findOrCreateTool(occmdService.getToolDto())
        findOrCreateTool(repositoryDetailsService.getToolDto())
    }

    @Transactional(readOnly = true)
    fun findToolByName(tool: CreateToolDto): ToolEntity? {
        return toolRepository.findByNameIgnoreCase(
            tool.name
        )
    }

    private fun findOrCreateTool(tool: CreateToolDto): ToolEntity {
        return toolRepository.findByNameIgnoreCase(
            tool.name
        )
            ?: toolRepository.saveAndFlush(tool.asDbObject())
    }
}
