package de.fraunhofer.iem.dataprovider.toolRun.service

import de.fraunhofer.iem.dataprovider.kpi.dto.RawValueKpiCreateDto
import de.fraunhofer.iem.dataprovider.repository.dto.RepositoryCreateDto
import de.fraunhofer.iem.dataprovider.repository.service.RepositoryService
import de.fraunhofer.iem.dataprovider.tool.dto.CreateToolDto
import de.fraunhofer.iem.dataprovider.tool.service.ToolService
import de.fraunhofer.iem.dataprovider.toolRun.dto.ToolRunDto
import de.fraunhofer.iem.dataprovider.toolRun.entity.LanguageEntity
import de.fraunhofer.iem.dataprovider.toolRun.entity.ToolRunEntity
import de.fraunhofer.iem.dataprovider.toolRun.repository.ToolRunRepository
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional

@Service
class ToolRunDbService(
    private val toolRunRepository: ToolRunRepository,
    private val repositoryService: RepositoryService,
    private val toolService: ToolService
) {

    @Transactional
    fun createToolRun(
        repoDto: RepositoryCreateDto,
        kpiToolList: List<Pair<CreateToolDto, List<RawValueKpiCreateDto>>>,
        languageMap: Map<String, Float>
    ) {
        val repo = repositoryService.getOrCreate(repoDto)
        val toolRun = ToolRunEntity(repository = repo)

        val languages =
            languageMap.map {
                LanguageEntity(
                    toolRunEntity = toolRun,
                    name = it.key,
                    percentage = it.value
                )
            }
        toolRun.languageEntities.addAll(languages)

        kpiToolList.forEach { toolKpi ->
            toolService.findToolByName(toolKpi.first)?.let { tool ->
                toolRun.toolEntities.add(tool)
            }

            val kpis = toolKpi.second.map { rawKpi ->
                rawKpi.toDbObject(toolRun)
            }
            toolRun.kpiEntities.addAll(kpis)
        }

        repo.toolRuns.add(toolRun)
        repositoryService.saveRepository(repo)
    }

    @Transactional(readOnly = true)
    fun getToolRunByProjectId(projectId: Long): ToolRunDto {
        return ToolRunDto.getDtoFromEntity(
            toolRunRepository.findFirstByRepository_ProjectIdOrderByCreatedAtDesc(projectId)
        )
    }
}
